import ckan.plugins.toolkit as toolkit
import requests


def service_available():
    url = toolkit.config.get("ckanext.chat.completion_url")
    if not url:
        return False
    try:
        # Perform a HEAD request (lightweight check) to see if the service responds
        response = requests.head(url, timeout=5, verify=False)
        if (200 <= response.status_code < 400) or response.status_code == 405:
            return True  # URL is reachable and returns a valid status code
        else:
            return False  # URL is reachable but response status is not valid
    except requests.RequestException as e:
        # If there's any issue (timeout, connection error, etc.)
        return False


def get_helpers():
    return {
        "service_available": service_available,
    }



import asyncio

import ckan.lib.base as base
import ckan.lib.helpers as core_helpers
import ckan.plugins.toolkit as toolkit
from ckan.common import _, current_user
from flask import Blueprint, current_app, jsonify, request
from flask.views import MethodView

from ckanext.chat.bot.agent import (Deps, async_agent_response,
                                    exception_to_model_response,
                                    user_input_to_model_request)
from ckanext.chat.helpers import service_available

blueprint = Blueprint("chat", __name__)

log = __import__("logging").getLogger(__name__)
global_ckan_app = None


@blueprint.before_request
def capture_global_app():
    # This hook is executed in an active application context.
    global global_ckan_app
    if global_ckan_app is None:
        # Capture the global CKAN app from the current request's context
        global_ckan_app = current_app._get_current_object()


class ChatView(MethodView):
    def post(self):
        return core_helpers.redirect_to(
            "chat.chat",
        )

    def get(self):
        if current_user.is_anonymous:
            core_helpers.flash_error(_("Not authorized to see this page"))

            # flask types do not mention that it's possible to return a response
            # from the `before_request` callback
            return core_helpers.redirect_to("user.login")
        # log.debug(get_ckan_url_patterns())
        return base.render(
            "chat/chat_ui.html",
            extra_vars={
                "service_status": service_available(),
                "token": toolkit.config.get("ckanext.chat.api_token"),
                "api_endpoint": toolkit.config.get("ckanext.chat.completion_url"),
            },
        )


from pydantic_ai.messages import ModelMessage, TextPart

# Assuming 'response' is an instance of ModelResponse

# Proceed with processing 'filtered_parts'


def ask():
    user_input = request.form.get("text")
    history = request.form.get("history", "")
    max_retries = 3
    attempt = 0
    tkuser = toolkit.current_user
    # If they're not a logged in user, don't allow them to see content
    if tkuser.name is None:
        return {"success": False, "msg": "Must be logged in to view site"}
    deps = Deps(user_id=tkuser.id)
    # log.debug(user)
    while attempt < max_retries:
        try:
            response = asyncio.run(async_agent_response(user_input, history, deps=deps))
            # Now response is guaranteed to have new_messages() if no exception occurred.
            # Ensure new_messages() is awaited in the sync wrapper if it's async
            messages = response.new_messages()
            # remove empty text responses parts
            [
                [
                    message.parts.remove(part)
                    for part in message.parts
                    if isinstance(part, TextPart) and part.content == ""
                ]
                for message in messages
            ]
            return jsonify({"response": messages})

        except Exception as e:
            user_promt = user_input_to_model_request(user_input)
            error_response = exception_to_model_response(e)
            log.error(error_response)
            return jsonify({"response": [user_promt, error_response]})


blueprint.add_url_rule(
    "/chat",
    view_func=ChatView.as_view(str("chat")),
)

blueprint.add_url_rule(
    "/chat/ask",
    view_func=ask,
    methods=["POST"],
)


def get_blueprint():
    return blueprint



import ckan.plugins as p


def chat_submit(context, data_dict, privilege="package_update"):
    user = context.get("user")
    if user and context.get("auth_user_obj"):
        return {"success": True}
    else:
        return {"success": False}


def get_auth_functions():
    return {
        "chat_submit": chat_submit,
    }



from typing import Any

import ckan.plugins as p
import ckan.plugins.toolkit as toolkit

if toolkit.check_ckan_version("2.10"):
    from ckan.types import Context
else:

    class Context(dict):
        def __init__(self, **kwargs):
            super().__init__(**kwargs)


log = __import__("logging").getLogger(__name__)


def chat_submit(context: Context, data_dict: dict[str, Any]) -> dict[str, Any]:
    """Send a chat message"""
    toolkit.check_access("chat_submit", context, data_dict)
    return True


def get_actions():
    actions = {
        "chat_submit": chat_submit,
    }
    return actions



import ckan.plugins as plugins
import ckan.plugins.toolkit as toolkit
from ckan.config.declaration import Declaration, Key
from ckan.lib.plugins import DefaultTranslation

from ckanext.chat import action, auth, helpers, views


class ChatPlugin(plugins.SingletonPlugin, DefaultTranslation):
    plugins.implements(plugins.IConfigurer)
    plugins.implements(plugins.IConfigDeclaration)
    plugins.implements(plugins.IBlueprint)
    plugins.implements(plugins.ITemplateHelpers)
    plugins.implements(plugins.IAuthFunctions)
    plugins.implements(plugins.IActions)
    plugins.implements(plugins.ITranslation)

    # IConfigurer

    def update_config(self, config_):
        toolkit.add_template_directory(config_, "templates")
        toolkit.add_public_directory(config_, "public")
        toolkit.add_resource("assets", "chat")

    # IConfigDeclaration

    def declare_config_options(self, declaration: Declaration, key: Key):

        declaration.annotate("chat")
        group = key.ckanext.chat
        declaration.declare_bool(group.ssl_verify, True)
        declaration.declare(group.completion_url, "https://your.chat.api")
        declaration.declare(group.deployment, "gpt-4-vision-preview")
        declaration.declare(group.api_token, "your-api-token")
        declaration.declare(group.embedding_model, 'text-embedding-3-small')
        declaration.declare(group.embedding_api, "")
        declaration.declare(group.collection_name, "")

    # IBlueprint

    def get_blueprint(self):
        return views.get_blueprint()

    # ITemplateHelpers

    def get_helpers(self):
        return helpers.get_helpers()

    # IAuthFunctions

    def get_auth_functions(self):
        return auth.get_auth_functions()

    # IActions

    def get_actions(self):
        actions = action.get_actions()
        return actions



# -*- coding: utf-8 -*-
from codecs import open  # To use a consistent encoding
from os import environ, path

from setuptools import (  # Always prefer setuptools over distutils
    find_packages, setup)

here = path.abspath(path.dirname(__file__))


# Get the long description from the relevant file
with open(path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

with open(path.join(here, "requirements.txt")) as f:
    requirements = f.read().splitlines()

setup(
    name="""ckanext-chat""",
    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # http://packaging.python.org/en/latest/tutorial.html#version
    version=environ.get("VERSION", "0.0.0"),
    description="""Extension adds a pydantic ai chat interface to CKAN, that can run actions with user aware context.""",
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=requirements,
    # The project's main homepage.
    url="https://github.com/Mat-O-Lab/ckanext-chat",
    # Author details
    author="""Thomas Hanke""",
    author_email="""thomas.hanke@iwm.fraunhofer.de""",
    # Choose your license
    license="AGPL",
    message_extractors={
        "ckanext": [
            ("**.py", "python", None),
            ("**.js", "javascript", None),
            ("**/templates/**.html", "ckan", None),
        ],
    },
    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    entry_points="""
        [ckan.plugins]
        chat=ckanext.chat.plugin:ChatPlugin
    """,
)
